# coding: utf-8

"""
Mozart platform API

API for interacting with the Mozart platform.

The version of the OpenAPI document: 0.2.0
Contact: support@bang-olufsen.dk
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import List, Optional

try:
    from pydantic.v1 import (
        BaseModel,
        Field,
        StrictBool,
        StrictInt,
        StrictStr,
        conlist,
        validator,
    )
except ImportError:
    from pydantic import (
        BaseModel,
        Field,
        StrictBool,
        StrictInt,
        StrictStr,
        conlist,
        validator,
    )

from mozart_api.models.art import Art
from mozart_api.models.beolink_leader import BeolinkLeader


class PlaybackContentMetadata(BaseModel):
    """
    PlaybackContentMetadata
    """

    album_name: Optional[StrictStr] = Field(default=None, alias="albumName")
    art: Optional[conlist(Art)] = None
    artist_name: Optional[StrictStr] = Field(default=None, alias="artistName")
    bitdepth: Optional[StrictInt] = None
    bitrate: Optional[StrictInt] = None
    container_name: Optional[StrictStr] = Field(default=None, alias="containerName")
    encoding: Optional[StrictStr] = None
    genre: Optional[StrictStr] = None
    id: Optional[StrictInt] = None
    input_channel_processing: Optional[StrictStr] = Field(
        default=None,
        alias="inputChannelProcessing",
        description="Input processing/decoding on top of the base codec indicated in encoding. E.g. Dolby Atmos on top of Dolby TrueHD or Dolby Surround on top of PCM. For Dolby, this value has priority over encoding, meaning that if inputChannelProcessing has a value, it must be indicated in the app, and optionally the encoding value can be indicated as well. If inputChannelProcessing does not have value the encoding value must be indicated. ",
    )
    input_channels: Optional[StrictStr] = Field(
        default=None, alias="inputChannels", description="e.g. 5.1"
    )
    is_explicit: Optional[StrictBool] = Field(default=None, alias="isExplicit")
    organization: Optional[StrictStr] = Field(
        default=None,
        description='This can be filled by gstreamer\'s GST_TAG_ORGANIZATION. Mozart can also fill this with netradio station name like "P3" and TV content like "Netflix". This is needed so it\'s possible to show who the provider is for the playing audio track/content. ',
    )
    output_channel_processing: Optional[StrictStr] = Field(
        default=None, alias="outputChannelProcessing", description="e.g. downmix"
    )
    output_channels: Optional[StrictStr] = Field(
        default=None, alias="outputChannels", description="e.g. 7.1"
    )
    queue_id: Optional[StrictStr] = Field(default=None, alias="queueId")
    remote_leader: Optional[BeolinkLeader] = Field(default=None, alias="remoteLeader")
    remote_source: Optional[StrictStr] = Field(default=None, alias="remoteSource")
    samplerate: Optional[StrictInt] = None
    source: Optional[StrictStr] = None
    source_internal_id: Optional[StrictStr] = Field(
        default=None, alias="sourceInternalId"
    )
    tags: Optional[conlist(StrictStr)] = None
    title: Optional[StrictStr] = None
    total_duration: Optional[StrictInt] = Field(
        default=None,
        alias="totalDuration",
        description="to be removed once all devices and apps are updated",
    )
    total_duration_seconds: Optional[StrictInt] = Field(
        default=None, alias="totalDurationSeconds"
    )
    track: Optional[StrictInt] = None
    track_count: Optional[StrictInt] = Field(default=None, alias="trackCount")
    uri: Optional[StrictStr] = None
    __properties = [
        "albumName",
        "art",
        "artistName",
        "bitdepth",
        "bitrate",
        "containerName",
        "encoding",
        "genre",
        "id",
        "inputChannelProcessing",
        "inputChannels",
        "isExplicit",
        "organization",
        "outputChannelProcessing",
        "outputChannels",
        "queueId",
        "remoteLeader",
        "remoteSource",
        "samplerate",
        "source",
        "sourceInternalId",
        "tags",
        "title",
        "totalDuration",
        "totalDurationSeconds",
        "track",
        "trackCount",
        "uri",
    ]

    @validator("encoding")
    def encoding_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in (
            "pcm",
            "wav",
            "mp3",
            "oggvorbis",
            "aac",
            "flac",
            "sbc",
            "alac",
            "opus",
            "DolbyDigital",
            "DolbyDigitalPlus",
            "DolbyTrueHD",
            "unknown",
        ):
            raise ValueError(
                "must be one of enum values ('pcm', 'wav', 'mp3', 'oggvorbis', 'aac', 'flac', 'sbc', 'alac', 'opus', 'DolbyDigital', 'DolbyDigitalPlus', 'DolbyTrueHD', 'unknown')"
            )
        return value

    @validator("input_channel_processing")
    def input_channel_processing_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in (
            "dolbySurround",
            "dolbyAtmos",
        ):
            raise ValueError(
                "must be one of enum values ('dolbySurround', 'dolbyAtmos')"
            )
        return value

    @validator("output_channel_processing")
    def output_channel_processing_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in (
            "Direct",
            "Downmix",
            "TrueImage",
        ):
            raise ValueError(
                "must be one of enum values ('Direct', 'Downmix', 'TrueImage')"
            )
        return value

    class Config:
        """Pydantic configuration"""

        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> PlaybackContentMetadata:
        """Create an instance of PlaybackContentMetadata from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True, exclude={}, exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of each item in art (list)
        _items = []
        if self.art:
            for _item in self.art:
                if _item:
                    _items.append(_item.to_dict())
            _dict["art"] = _items
        # override the default output from pydantic by calling `to_dict()` of remote_leader
        if self.remote_leader:
            _dict["remoteLeader"] = self.remote_leader.to_dict()
        # set to None if input_channel_processing (nullable) is None
        # and __fields_set__ contains the field
        if (
            self.input_channel_processing is None
            and "input_channel_processing" in self.__fields_set__
        ):
            _dict["inputChannelProcessing"] = None

        # set to None if input_channels (nullable) is None
        # and __fields_set__ contains the field
        if self.input_channels is None and "input_channels" in self.__fields_set__:
            _dict["inputChannels"] = None

        # set to None if output_channel_processing (nullable) is None
        # and __fields_set__ contains the field
        if (
            self.output_channel_processing is None
            and "output_channel_processing" in self.__fields_set__
        ):
            _dict["outputChannelProcessing"] = None

        # set to None if output_channels (nullable) is None
        # and __fields_set__ contains the field
        if self.output_channels is None and "output_channels" in self.__fields_set__:
            _dict["outputChannels"] = None

        # set to None if remote_source (nullable) is None
        # and __fields_set__ contains the field
        if self.remote_source is None and "remote_source" in self.__fields_set__:
            _dict["remoteSource"] = None

        # set to None if total_duration (nullable) is None
        # and __fields_set__ contains the field
        if self.total_duration is None and "total_duration" in self.__fields_set__:
            _dict["totalDuration"] = None

        # set to None if total_duration_seconds (nullable) is None
        # and __fields_set__ contains the field
        if (
            self.total_duration_seconds is None
            and "total_duration_seconds" in self.__fields_set__
        ):
            _dict["totalDurationSeconds"] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> PlaybackContentMetadata:
        """Create an instance of PlaybackContentMetadata from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return PlaybackContentMetadata.parse_obj(obj)

        _obj = PlaybackContentMetadata.parse_obj(
            {
                "album_name": obj.get("albumName"),
                "art": [Art.from_dict(_item) for _item in obj.get("art")]
                if obj.get("art") is not None
                else None,
                "artist_name": obj.get("artistName"),
                "bitdepth": obj.get("bitdepth"),
                "bitrate": obj.get("bitrate"),
                "container_name": obj.get("containerName"),
                "encoding": obj.get("encoding"),
                "genre": obj.get("genre"),
                "id": obj.get("id"),
                "input_channel_processing": obj.get("inputChannelProcessing"),
                "input_channels": obj.get("inputChannels"),
                "is_explicit": obj.get("isExplicit"),
                "organization": obj.get("organization"),
                "output_channel_processing": obj.get("outputChannelProcessing"),
                "output_channels": obj.get("outputChannels"),
                "queue_id": obj.get("queueId"),
                "remote_leader": BeolinkLeader.from_dict(obj.get("remoteLeader"))
                if obj.get("remoteLeader") is not None
                else None,
                "remote_source": obj.get("remoteSource"),
                "samplerate": obj.get("samplerate"),
                "source": obj.get("source"),
                "source_internal_id": obj.get("sourceInternalId"),
                "tags": obj.get("tags"),
                "title": obj.get("title"),
                "total_duration": obj.get("totalDuration"),
                "total_duration_seconds": obj.get("totalDurationSeconds"),
                "track": obj.get("track"),
                "track_count": obj.get("trackCount"),
                "uri": obj.get("uri"),
            }
        )
        return _obj



# coding: utf-8

"""
Mozart platform API

API for interacting with the Mozart platform.

The version of the OpenAPI document: 0.2.0
Contact: support@bang-olufsen.dk
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Optional, Union

try:
    from pydantic.v1 import (
        BaseModel,
        Field,
        StrictBool,
        StrictFloat,
        StrictInt,
        StrictStr,
        validator,
    )
except ImportError:
    from pydantic import (
        BaseModel,
        Field,
        StrictBool,
        StrictFloat,
        StrictInt,
        StrictStr,
        validator,
    )


class StandMovement(BaseModel):
    """
    StandMovement
    """

    angle: Optional[Union[StrictFloat, StrictInt]] = None
    ignore_endstop: Optional[StrictBool] = Field(default=None, alias="ignoreEndstop")
    stand_motion: Optional[StrictStr] = Field(default=None, alias="standMotion")
    __properties = ["angle", "ignoreEndstop", "standMotion"]

    @validator("stand_motion")
    def stand_motion_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in (
            "continuousCCW",
            "stepCCW",
            "stop",
            "stepCW",
            "continuousCW",
        ):
            raise ValueError(
                "must be one of enum values ('continuousCCW', 'stepCCW', 'stop', 'stepCW', 'continuousCW')"
            )
        return value

    class Config:
        """Pydantic configuration"""

        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> StandMovement:
        """Create an instance of StandMovement from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True, exclude={}, exclude_none=True)
        # set to None if angle (nullable) is None
        # and __fields_set__ contains the field
        if self.angle is None and "angle" in self.__fields_set__:
            _dict["angle"] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> StandMovement:
        """Create an instance of StandMovement from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return StandMovement.parse_obj(obj)

        _obj = StandMovement.parse_obj(
            {
                "angle": obj.get("angle"),
                "ignore_endstop": obj.get("ignoreEndstop"),
                "stand_motion": obj.get("standMotion"),
            }
        )
        return _obj



# coding: utf-8

"""
Mozart platform API

API for interacting with the Mozart platform.

The version of the OpenAPI document: 0.2.0
Contact: support@bang-olufsen.dk
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import List, Optional

try:
    from pydantic.v1 import BaseModel, Field, StrictStr, conlist, validator
except ImportError:
    from pydantic import BaseModel, Field, StrictStr, conlist, validator

from mozart_api.models.source_type_enum import SourceTypeEnum


class ContentItem(BaseModel):
    """
    ContentItem
    """

    categories: Optional[conlist(StrictStr)] = None
    content_uri: StrictStr = Field(default=..., alias="contentUri")
    label: Optional[StrictStr] = None
    source: SourceTypeEnum = Field(...)
    __properties = ["categories", "contentUri", "label", "source"]

    @validator("categories")
    def categories_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        for i in value:
            if i not in (
                "music",
                "movie",
                "tv",
                "hdmi",
                "app",
            ):
                raise ValueError(
                    "each list item must be one of ('music', 'movie', 'tv', 'hdmi', 'app')"
                )
        return value

    class Config:
        """Pydantic configuration"""

        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> ContentItem:
        """Create an instance of ContentItem from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True, exclude={}, exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of source
        if self.source:
            _dict["source"] = self.source.to_dict()
        # set to None if label (nullable) is None
        # and __fields_set__ contains the field
        if self.label is None and "label" in self.__fields_set__:
            _dict["label"] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> ContentItem:
        """Create an instance of ContentItem from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return ContentItem.parse_obj(obj)

        _obj = ContentItem.parse_obj(
            {
                "categories": obj.get("categories"),
                "content_uri": obj.get("contentUri"),
                "label": obj.get("label"),
                "source": SourceTypeEnum.from_dict(obj.get("source"))
                if obj.get("source") is not None
                else None,
            }
        )
        return _obj



# coding: utf-8

"""
Mozart platform API

API for interacting with the Mozart platform.

The version of the OpenAPI document: 0.2.0
Contact: support@bang-olufsen.dk
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from datetime import datetime
from typing import List, Optional

try:
    from pydantic.v1 import (
        BaseModel,
        Field,
        StrictBool,
        StrictStr,
        conint,
        conlist,
        validator,
    )
except ImportError:
    from pydantic import (
        BaseModel,
        Field,
        StrictBool,
        StrictStr,
        conint,
        conlist,
        validator,
    )


class PairedRemote(BaseModel):
    """
    PairedRemote
    """

    address: StrictStr = Field(...)
    app_version: Optional[StrictStr] = Field(default=None, alias="appVersion")
    battery_level: Optional[conint(strict=True, le=100, ge=0)] = Field(
        default=None, alias="batteryLevel"
    )
    connected: Optional[StrictBool] = None
    db_version: Optional[StrictStr] = Field(default=None, alias="dbVersion")
    last_seen: Optional[datetime] = Field(default=None, alias="lastSeen")
    name: StrictStr = Field(...)
    serial_number: Optional[StrictStr] = Field(default=None, alias="serialNumber")
    updated: Optional[conlist(StrictStr)] = None
    __properties = [
        "address",
        "appVersion",
        "batteryLevel",
        "connected",
        "dbVersion",
        "lastSeen",
        "name",
        "serialNumber",
        "updated",
    ]

    @validator("updated")
    def updated_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        for i in value:
            if i not in (
                "app",
                "db",
            ):
                raise ValueError("each list item must be one of ('app', 'db')")
        return value

    class Config:
        """Pydantic configuration"""

        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> PairedRemote:
        """Create an instance of PairedRemote from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True, exclude={}, exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> PairedRemote:
        """Create an instance of PairedRemote from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return PairedRemote.parse_obj(obj)

        _obj = PairedRemote.parse_obj(
            {
                "address": obj.get("address"),
                "app_version": obj.get("appVersion"),
                "battery_level": obj.get("batteryLevel"),
                "connected": obj.get("connected"),
                "db_version": obj.get("dbVersion"),
                "last_seen": obj.get("lastSeen"),
                "name": obj.get("name"),
                "serial_number": obj.get("serialNumber"),
                "updated": obj.get("updated"),
            }
        )
        return _obj



# coding: utf-8

"""
Mozart platform API

API for interacting with the Mozart platform.

The version of the OpenAPI document: 0.2.0
Contact: support@bang-olufsen.dk
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Optional, Union

try:
    from pydantic.v1 import BaseModel, Field, StrictStr, confloat, conint
except ImportError:
    from pydantic import BaseModel, Field, StrictStr, confloat, conint

from mozart_api.models.speaker_group_member_location import SpeakerGroupMemberLocation


class SpeakerGroupMember(BaseModel):
    """
    SpeakerGroupMember
    """

    distance: Optional[conint(strict=True, le=1500, ge=0)] = Field(
        default=None, description="Distance from listening position in cm"
    )
    friendly_name: Optional[StrictStr] = Field(default=None, alias="friendlyName")
    gain: Optional[
        Union[confloat(le=6, ge=-24, strict=True), conint(le=6, ge=-24, strict=True)]
    ] = Field(default=None, description="Level in dB")
    id: StrictStr = Field(...)
    location: Optional[SpeakerGroupMemberLocation] = None
    redirection_level: Optional[conint(strict=True, le=6, ge=-100)] = Field(
        default=None,
        alias="redirectionLevel",
        description="Redirection level for bass management in dB",
    )
    role: Optional[StrictStr] = None
    type: Optional[StrictStr] = None
    __properties = [
        "distance",
        "friendlyName",
        "gain",
        "id",
        "location",
        "redirectionLevel",
        "role",
        "type",
    ]

    class Config:
        """Pydantic configuration"""

        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> SpeakerGroupMember:
        """Create an instance of SpeakerGroupMember from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True, exclude={}, exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of location
        if self.location:
            _dict["location"] = self.location.to_dict()
        # set to None if distance (nullable) is None
        # and __fields_set__ contains the field
        if self.distance is None and "distance" in self.__fields_set__:
            _dict["distance"] = None

        # set to None if friendly_name (nullable) is None
        # and __fields_set__ contains the field
        if self.friendly_name is None and "friendly_name" in self.__fields_set__:
            _dict["friendlyName"] = None

        # set to None if gain (nullable) is None
        # and __fields_set__ contains the field
        if self.gain is None and "gain" in self.__fields_set__:
            _dict["gain"] = None

        # set to None if location (nullable) is None
        # and __fields_set__ contains the field
        if self.location is None and "location" in self.__fields_set__:
            _dict["location"] = None

        # set to None if redirection_level (nullable) is None
        # and __fields_set__ contains the field
        if (
            self.redirection_level is None
            and "redirection_level" in self.__fields_set__
        ):
            _dict["redirectionLevel"] = None

        # set to None if role (nullable) is None
        # and __fields_set__ contains the field
        if self.role is None and "role" in self.__fields_set__:
            _dict["role"] = None

        # set to None if type (nullable) is None
        # and __fields_set__ contains the field
        if self.type is None and "type" in self.__fields_set__:
            _dict["type"] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> SpeakerGroupMember:
        """Create an instance of SpeakerGroupMember from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return SpeakerGroupMember.parse_obj(obj)

        _obj = SpeakerGroupMember.parse_obj(
            {
                "distance": obj.get("distance"),
                "friendly_name": obj.get("friendlyName"),
                "gain": obj.get("gain"),
                "id": obj.get("id"),
                "location": SpeakerGroupMemberLocation.from_dict(obj.get("location"))
                if obj.get("location") is not None
                else None,
                "redirection_level": obj.get("redirectionLevel"),
                "role": obj.get("role"),
                "type": obj.get("type"),
            }
        )
        return _obj



"""Main Mozart CLI document."""

import argparse
import asyncio
import ipaddress
import re
import sys
import threading
from dataclasses import dataclass
from typing import Final, cast

from aioconsole import ainput  # type: ignore[import-untyped]
from zeroconf import ServiceBrowser, ServiceListener, Zeroconf

from mozart_api import __version__
from mozart_api.models import BeolinkJoinRequest, VolumeLevel, VolumeMute
from mozart_api.mozart_client import MozartClient, WebSocketEventType

MOZART_MDNS_TYPE: Final[str] = "_bangolufsen._tcp.local."
MDNS_TIMEOUT: Final[int] = 10
DISCOVER_MODE: Final[str] = "discover"
VERSION_MODE: Final[str] = "version"

AVAILABLE_COMMANDS: Final[list[str]] = [
    "preset",
    "play",
    "pause",
    "next",
    "previous",
    "mute",
    "unmute",
    "volume",
    "join",
    "info",
    "allstandby",
]


@dataclass
class MozartDevice:
    """Dataclass for describing Zeroconf discovered Mozart devices."""

    friendly_name: str | None = ""
    model_name: str = ""
    serial_number: str = ""
    ip_address: str = ""
    sw_version: str = ""


mozart_devices: list[MozartDevice] = []


class MozartListener(ServiceListener):
    """Listener for Zeroconf discovery of Mozart devices."""

    def __init__(self, mode: str, verbose: bool, event: threading.Event) -> None:
        """Initialize listener."""
        super().__init__()
        self.mode = mode
        self.verbose = verbose
        self.event = event

    def update_service(self, zc: Zeroconf, type_: str, name: str) -> None:
        """Unused."""

    def remove_service(self, zc: Zeroconf, type_: str, name: str) -> None:
        """Unused."""

    def add_service(self, zc: Zeroconf, type_: str, name: str) -> None:
        """Add discovered Mozart device."""
        info = zc.get_service_info(type_, name)

        # Sometimes service info is None.
        if not info:
            print(f"Error getting {name}")
            return

        # Create MozartDevice object from MDNS discovered information.
        ip_address = info.parsed_addresses()[0]
        serial_number = cast(bytes, info.properties.get(b"sn")).decode("utf-8")
        model_name = cast(str, info.server)[:-16].replace("-", " ")
        sw_version = cast(bytes, info.properties.get(b"fv")).decode("utf-8")

        friendly_name = info.properties.get(b"fn")

        mozart_device = MozartDevice(
            friendly_name.decode("utf-8") if friendly_name is not None else None,
            model_name,
            serial_number,
            ip_address,
            sw_version,
        )

        mozart_devices.append(mozart_device)

        # Stop discovery if the desired Mozart device has been found.
        if self.mode == serial_number:
            print(f"Desired Mozart device: {self.mode} found: {mozart_device}")
            self.event.set()

        # Only print the discovered devices if in 'discover' mode or verbose is enabled.
        elif self.mode == DISCOVER_MODE or self.verbose:
            print(mozart_device)


def discover_devices(mode: str, timeout: int, verbose: bool) -> list[MozartDevice]:
    """MDNS discovery of devices on the current network."""
    event = threading.Event()

    zeroconf = Zeroconf()
    listener = MozartListener(mode, verbose, event)
    browser = ServiceBrowser(zeroconf, MOZART_MDNS_TYPE, listener)

    if mode == "discover" or verbose:
        print("Discovering Mozart devices. Scanning _bangolufsen._tcp.local.")

    if timeout == -1:
        input("Press 'enter' to stop discovery.\n\r")

    else:
        # Stop if the serial number has been found with MDNS
        timeout_status = event.wait(timeout)

        if not timeout_status:
            print(f"Discovery timed out with timeout of {timeout} seconds.")

    browser.cancel()
    zeroconf.close()

    return mozart_devices


def init_argument_parser() -> argparse.ArgumentParser:
    """Initialize  and add arguments."""
    parser = argparse.ArgumentParser(
        prog="mozart_api",
        description="CLI for sending simple commands to Mozart devices.",
    )

    parser.add_argument(
        "--verbose",
        "-v",
        action="store_true",
        help="Specify if the output should be verbose.",
    )

    parser.add_argument(
        "--websocket",
        "-w",
        action="store_true",
        help="Specify if the websocket listener should be active.",
    )

    parser.add_argument(
        "--timeout",
        "-t",
        action="store",
        help="""Specify Zeroconf discovery timeout.
                '-1' for input-interrupted discovery in 'discover' mode.""",
    )

    parser.add_argument(
        "mode",
        choices=(DISCOVER_MODE, VERSION_MODE),
        help=f"""Specify the serial number or IP address for command execution
                ,'{DISCOVER_MODE}' for Zeroconf discovery of Mozart devices or "{VERSION_MODE}" to get the API client version.""",
    )

    parser.add_argument(
        "command",
        nargs="?",
        choices=AVAILABLE_COMMANDS,
        help="Specify the command.",
    )

    parser.add_argument(
        "command_args",
        nargs="*",
        help="Specify the command arguments if needed.",
    )
    return parser


class MozartApiCli:
    """Mozart CLI client."""

    def __init__(self) -> None:
        """Init the Mozart CLI."""
        self.timeout = MDNS_TIMEOUT
        self.verbose = False
        self.websocket = False
        self.mode = ""
        self.command = ""
        self.host = ""
        self.command_args: list[str] = []
        self.mozart_devices: list[MozartDevice] = []

        parser = init_argument_parser()
        args = parser.parse_args()

        if args.timeout:
            self.timeout = int(args.timeout)

        self.verbose = bool(args.verbose)
        self.websocket = bool(args.websocket)
        self.mode = args.mode
        self.command = args.command
        self.command_args = args.command_args

        # Check if the API version should be printed
        if self.mode == VERSION_MODE:
            print(__version__)
            sys.exit(0)

        # Check if the mode defined is an ip address
        try:
            ipaddress.ip_address(self.mode)
            self.host = self.mode
        except ValueError as exception:
            # Ensure that the mode's serial number
            # has the correct format or 'discover' mode.
            if self.mode != DISCOVER_MODE and re.fullmatch(r"\d{8}", self.mode) is None:
                # Check if the mode is then an ip address
                msg = f""""{self.mode}" has an invalid value.
                    Must either be a serial number, ip address, "{DISCOVER_MODE}" or "{VERSION_MODE}"."""
                raise ValueError(msg) from exception

        # Discover devices if host has not been defined
        if not self.host:
            self.mozart_devices = discover_devices(
                self.mode,
                self.timeout,
                self.verbose,
            )

            # Get the ip address from the devices Mozart devices
            self.host = next(
                (
                    device
                    for device in self.mozart_devices
                    if device.serial_number == self.mode
                ),
                MozartDevice(),
            ).ip_address

        # Exit if in discover mode, no command has been defined
        # or desired host can't be found.
        if self.mode == DISCOVER_MODE or self.command == "" or self.host == "":
            sys.exit(0)

        asyncio.run(self._run_api())

        sys.exit(0)

    async def _run_api(self) -> None:
        """Run async API command handling."""
        # Generate MozartApi object for calling API endpoints.
        self.mozart_client = MozartClient(self.host)

        # Connect to the websocket notification channel if defined
        if self.websocket:
            print("Connecting to WebSocket channel")
            self.mozart_client.get_all_notifications(self.all_notifications)
            await self.mozart_client.connect_notifications(remote_control=True)

        # Handle command
        await self._command_handler()

        # If WebSocket listener is enabled,
        # then wait for keypress before exiting the CLI
        if self.websocket:
            await ainput(
                "Listening to WebSocket events. Press any key to exit CLI.\n\r",
            )
            self.mozart_client.disconnect_notifications()

        await self.mozart_client.close_api_client()

    async def _beolink_join(self) -> BeolinkJoinRequest | None:
        """Showcase async API usage of the Beolink command."""
        # If no JID is specified, then join an active experience if available
        if len(self.command_args) == 0:
            status = await self.mozart_client.join_latest_beolink_experience()

        else:
            serial_number = self.command_args[0]

            # Check if a device with specified serial number
            # is available as a peer and get JID if available
            peers = await self.mozart_client.get_beolink_peers()

            # The peers may be outdated and still have now unavailable devices.
            if len(peers) == 0:
                print("No available Beolink peers.")
                return None

            jid = next(peer for peer in peers if serial_number in peer.jid).jid

            status = await self.mozart_client.join_beolink_peer(jid=jid)

        return status

    def all_notifications(
        self,
        notification: WebSocketEventType,
        notification_type: str,
    ) -> None:
        """Handle all notifications."""
        print(notification_type, notification)

    async def _command_handler(self) -> None:
        """Handle commands."""
        print(
            f"Sending command: '{self.command}' to device with args {self.command_args}.",
        )
        status = None

        if self.command == "preset":
            preset_id = int(self.command_args[0])
            await self.mozart_client.activate_preset(id=preset_id)

        elif self.command in ("play", "pause", "next", "previous"):
            await self.mozart_client.post_playback_command(command=self.command)

        elif self.command == "mute":
            await self.mozart_client.set_volume_mute(volume_mute=VolumeMute(muted=True))

        elif self.command == "unmute":
            await self.mozart_client.set_volume_mute(
                volume_mute=VolumeMute(muted=False),
            )

        elif self.command == "volume":
            volume_level = int(self.command_args[0])
            await self.mozart_client.set_current_volume_level(
                volume_level=VolumeLevel(level=volume_level),
            )

        elif self.command == "join":
            status = await self._beolink_join()

        # Currently show battery state, product state
        elif self.command == "info":
            battery_state = await self.mozart_client.get_battery_state()
            print(f"Battery state: {battery_state}")

            power_state = await self.mozart_client.get_product_state()
            print(f"Product state: {power_state}")

        elif self.command == "allstandby":
            await self.mozart_client.post_beolink_allstandby()

        else:
            print(f"Invalid command {self.command}.")
            return

        # Print verbose status information if defined.
        if self.verbose and self.command == "join":
            # Wait for the join-result to be available
            await asyncio.sleep(1)
            if status:
                print("Beolink Join status:")
                print(
                    await self.mozart_client.get_beolink_join_result(
                        id=status.request_id,
                    ),
                )



# coding: utf-8

"""
Mozart platform API

API for interacting with the Mozart platform.

The version of the OpenAPI document: 0.2.0
Contact: support@bang-olufsen.dk
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Optional

try:
    from pydantic.v1 import BaseModel, Field, conint
except ImportError:
    from pydantic import BaseModel, Field, conint

from mozart_api.models.video_pixel_format import VideoPixelFormat
from mozart_api.models.video_timings import VideoTimings


class HdmiVideoFormat(BaseModel):
    """
    HdmiVideoFormat
    """

    pixel_format: Optional[VideoPixelFormat] = Field(default=None, alias="pixelFormat")
    v_ic: Optional[conint(strict=True, ge=0)] = Field(default=None, alias="vIC")
    video_timings: Optional[VideoTimings] = Field(default=None, alias="videoTimings")
    __properties = ["pixelFormat", "vIC", "videoTimings"]

    class Config:
        """Pydantic configuration"""

        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> HdmiVideoFormat:
        """Create an instance of HdmiVideoFormat from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True, exclude={}, exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of pixel_format
        if self.pixel_format:
            _dict["pixelFormat"] = self.pixel_format.to_dict()
        # override the default output from pydantic by calling `to_dict()` of video_timings
        if self.video_timings:
            _dict["videoTimings"] = self.video_timings.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> HdmiVideoFormat:
        """Create an instance of HdmiVideoFormat from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return HdmiVideoFormat.parse_obj(obj)

        _obj = HdmiVideoFormat.parse_obj(
            {
                "pixel_format": VideoPixelFormat.from_dict(obj.get("pixelFormat"))
                if obj.get("pixelFormat") is not None
                else None,
                "v_ic": obj.get("vIC"),
                "video_timings": VideoTimings.from_dict(obj.get("videoTimings"))
                if obj.get("videoTimings") is not None
                else None,
            }
        )
        return _obj



# coding: utf-8

"""
Mozart platform API

API for interacting with the Mozart platform.

The version of the OpenAPI document: 0.2.0
Contact: support@bang-olufsen.dk
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Optional

try:
    from pydantic.v1 import BaseModel, Field, StrictStr, validator
except ImportError:
    from pydantic import BaseModel, Field, StrictStr, validator


class BeolinkJoinResult(BaseModel):
    """
    BeolinkJoinResult
    """

    error: Optional[StrictStr] = Field(default=None, description="Join error")
    jid: StrictStr = Field(default=..., description="Beolink peer ID")
    request_id: StrictStr = Field(
        default=..., alias="requestID", description="Beolink request ID"
    )
    status: StrictStr = Field(default=..., description="Overall join status")
    type: StrictStr = Field(
        default=..., description="The type of join request this is a result of"
    )
    __properties = ["error", "jid", "requestID", "status", "type"]

    @validator("error")
    def error_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in (
            "ok",
            "busy",
            "leaderless",
            "alone",
            "silent",
            "missing",
            "rejected",
            "timeout",
        ):
            raise ValueError(
                "must be one of enum values ('ok', 'busy', 'leaderless', 'alone', 'silent', 'missing', 'rejected', 'timeout')"
            )
        return value

    @validator("status")
    def status_validate_enum(cls, value):
        """Validates the enum"""
        if value not in (
            "idle",
            "joined",
            "failed",
        ):
            raise ValueError("must be one of enum values ('idle', 'joined', 'failed')")
        return value

    @validator("type")
    def type_validate_enum(cls, value):
        """Validates the enum"""
        if value not in (
            "join",
            "joinRequested",
            "expand",
            "expandRequested",
        ):
            raise ValueError(
                "must be one of enum values ('join', 'joinRequested', 'expand', 'expandRequested')"
            )
        return value

    class Config:
        """Pydantic configuration"""

        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> BeolinkJoinResult:
        """Create an instance of BeolinkJoinResult from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True, exclude={}, exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> BeolinkJoinResult:
        """Create an instance of BeolinkJoinResult from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return BeolinkJoinResult.parse_obj(obj)

        _obj = BeolinkJoinResult.parse_obj(
            {
                "error": obj.get("error"),
                "jid": obj.get("jid"),
                "request_id": obj.get("requestID"),
                "status": obj.get("status"),
                "type": obj.get("type"),
            }
        )
        return _obj
